@@ -155,6 +155,8 @@ class TopicManager:
     a task.
     """
 
+    __slots__ = ('_loop', '_topic')
+
     def __init__(self, topic, *, loop=None):
         self._loop = loop or asyncio.get_event_loop()
         self._topic = topic
@@ -172,7 +174,7 @@ class TopicManager:
         loop = cls._loop if isinstance(cls, TopicManager) else None
         if isinstance(cls, TopicManager):
             topic = cls._topic
-        coro = asyncio.Task.current_task(loop=loop).data_received
+        coro = asyncio.current_task(loop=loop).data_received
         get_broker(loop=loop).register(coro, topic=topic)
 
     def __enter__(self):
@@ -191,7 +193,7 @@ class TopicManager:
         loop = cls._loop if isinstance(cls, TopicManager) else None
         if isinstance(cls, TopicManager):
             topic = cls._topic
-        coro = asyncio.Task.current_task(loop=loop).data_received
+        coro = asyncio.current_task(loop=loop).data_received
         get_broker(loop=loop).unregister(coro, topic=topic)
 
     def __exit__(self, *exc):
