@@ -100,10 +100,10 @@ class _WorkflowSelector:
 
     def get(self, tmpl_id):
         """
-        Returns the workflow template with the given template ID. Raises a
-        `KeyError` exception if no template is found.
+        Returns the workflow template with the given template ID, None if
+        no template is found.
         """
-        return self._templates[tmpl_id]
+        return self._templates.get(tmpl_id)
 
 
 class Engine(asyncio.Future):
@@ -117,11 +117,11 @@ class Engine(asyncio.Future):
     running workflows are done. Afterwards no new workflow can be triggered.
     """
 
-    def __init__(self, *, loop=None):
+    def __init__(self, *, selector=None, loop=None):
         super().__init__(loop=loop)
         # use the custom asyncio task factory
         self._loop.set_task_factory(tukio_factory)
-        self._selector = _WorkflowSelector()
+        self._selector = selector or _WorkflowSelector()
         self._instances = []
         self._broker = get_broker(self._loop)
         self._lock = asyncio.Lock()
@@ -158,9 +158,9 @@ class Engine(asyncio.Future):
         try:
             wflow.result()
         except asyncio.CancelledError:
-            log.warning('workflow %s has been cancelled', wflow)
+            log.info('Workflow %s has been cancelled', wflow.uid[:8])
         except Exception as exc:
-            log.warning('workflow %s ended on exception', wflow)
+            log.warning('Workflow %s ended on exception', wflow.uid[:8])
             log.exception(exc)
 
         if self._must_stop and not self._instances and not self.done():
@@ -181,16 +181,6 @@ class Engine(asyncio.Future):
                 log.debug('cancelled workflow %s', wflow)
         return self
 
-    def _run_in_task(self, callback, *args, **kwargs):
-        """
-        Wrap a regular function into a coroutine and run it in a task.
-        This is intended to wrap time consuming functions into a task so as to
-        prevent slow operations from blocking the whole loop.
-        """
-        async def coro():
-            return callback(*args, **kwargs)
-        return asyncio.ensure_future(coro(), loop=self._loop)
-
     def _load(self, template):
         """
         Loads a workflow template into the engine. Each workflow may be
@@ -208,7 +198,7 @@ class Engine(asyncio.Future):
         coroutines from updating the dict of loaded templates in the mean time.
         """
         with await self._lock:
-            await self._run_in_task(self._load, template)
+            self._load(template)
 
     async def reload(self, templates):
         """
@@ -218,7 +208,7 @@ class Engine(asyncio.Future):
         with await self._lock:
             self._selector.clear()
             for tmpl in templates:
-                await self._run_in_task(self._load, tmpl)
+                self._load(tmpl)
 
     async def unload(self, template_id):
         """
@@ -245,7 +235,10 @@ class Engine(asyncio.Future):
             log.debug("The engine is stopping, cannot trigger new workflows")
             return
         with await self._lock:
-            templates = self._selector.select(topic)
+            if asyncio.iscoroutinefunction(self._selector.select):
+                templates = await self._selector.select(topic)
+            else:
+                templates = self._selector.select(topic)
             # Try to trigger new workflows from the current dict of workflow
             # templates at all times!
             wflows = []
@@ -263,10 +256,11 @@ class Engine(asyncio.Future):
         """
         with await self._lock:
             # Ignore unknown (aka not loaded) workflow templates
-            try:
+            if asyncio.iscoroutinefunction(self._selector.get):
+                template = await self._selector.get(template_id)
+            else:
                 template = self._selector.get(template_id)
-            except KeyError:
-                log.error('workflow template %s not loaded', template_id)
+            if not template:
                 return None
             return self._try_run(template, Event(data))
 
@@ -290,34 +284,24 @@ class Engine(asyncio.Future):
             return
         # Always apply the policy of the current workflow template (workflow
         # instances may run with another version of the template)
-        running = [i for i in self._instances
-                   if i.template.uid == template.uid]
-        wflow = new_workflow(template, running=running, loop=self._loop)
+        wflow = new_workflow(template, running=self._instances, loop=self._loop)
         if wflow:
-            if template.policy == OverrunPolicy.abort_running and running:
-                def cb():
-                    self._do_run(wflow, event)
-                asyncio.ensure_future(self._wait_abort(running, cb))
-            else:
-                self._do_run(wflow, event)
+            if template.policy == OverrunPolicy.ABORT_RUNNING:
+                others = [
+                    i for i in self._instances
+                    if i.template.uid == template.uid
+                ]
+                if others:
+                    def cb(f):
+                        self._do_run(wflow, event)
+                    others_done = asyncio.ensure_future(asyncio.wait(others))
+                    others_done.add_done_callback(cb)
+                    return wflow
+            self._do_run(wflow, event)
         else:
             log.debug("skip new workflow from %s (overrun policy)", template)
         return wflow
 
-    async def _wait_abort(self, running, callback):
-        """
-        Wait for the end of a list of aborted (cancelled) workflows before
-        starting a new one when the policy is 'abort-running'.
-
-        XXX: we shouldn't find policy-specific code in the engine! Find a
-        better way to do it.
-        """
-        # Always act on a snapshot of the original running list. Don't forget
-        # it is a living list!
-        others = list(running)
-        await asyncio.wait(others)
-        callback()
-
     async def run_once(self, template, data):
         """
         Starts a new execution of the workflow template regardless of the
