@@ -11,7 +11,8 @@ import asyncio
 import logging
 import inspect
 
-from tukio.broker import get_broker, EXEC_TOPIC
+from tukio.event import Event
+from tukio.utils import TimeoutHandle
 
 
 log = logging.getLogger(__name__)
@@ -81,7 +82,7 @@ def register(task_name, coro_name=None):
     return decorator
 
 
-def new_task(task_name, *, data=None, config=None, loop=None):
+def new_task(task_name, *, data=None, config=None, timeout=None, loop=None):
     """
     Schedules the execution of the coroutine registered as `task_name` (either
     defined in a task holder class or not) in the loop and returns an instance
@@ -93,4 +94,8 @@ def new_task(task_name, *, data=None, config=None, loop=None):
         coro = coro_fn(task_holder, data)
     else:
         coro = coro_fn(data)
-    return asyncio.ensure_future(coro, loop=loop)
+    task = asyncio.ensure_future(coro, loop=loop)
+    task.inputs = data.data if isinstance(data, Event) else data
+    if timeout:
+        TimeoutHandle(task, timeout).start()
+    return task
